var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { enableScripts, getCookieValue, waitForWindowVariable } from '@rh-support/utils';
import { useEffect, useState } from 'react';
import { extraPreChatInfo } from '../components/EmbeddedServiceChat/ExtraPreChatInfo';
import { useChatConfig } from './useChatConfig';
import { usePreChatFormDetails } from './usePreChatFormDetails';
/**
 * This hook is used for setting chat up and listing custom event of chat.
 * for more details https://developer.salesforce.com/docs/atlas.en-us.snapins_web_dev.meta/snapins_web_dev/snapins_web_chat_events.htm
 * @param initialState
 * @returns
 */
export const useChatInit = (initialState = false) => {
    const [isChatStarted, setChatStart] = useState(initialState);
    // Need to set loading status true as it will prevent starting chat before loading necessary dependencies
    const [loadingChat, setLoadingChat] = useState(true);
    const [hasBlockedByBrowser, setHasBlockedByBrowser] = useState(false);
    const [hasChatDomainsBlocked, setHasChatDomainsBlocked] = useState(false);
    const [chatServiceNotAvailable, setChatServiceNotAvailable] = useState(false);
    const [hasBlockedByCookie, setHasBlockedByCookie] = useState(false);
    const getChatFormDetails = usePreChatFormDetails();
    const { host, sandBox, gslbBaseURL, key, appName, extra } = useChatConfig();
    useEffect(() => {
        /*
        retrieve TrustArc cookie value
        value is like "permit 1,2,3" when all cookie types are accepted
        1-> Required Cookies
        2-> Functional Cookies
        3-> Advertising Cookies
        we need to make sure 2 that indicates functional cookies is included in value
        */
        var preferences = getCookieValue('cmapi_cookie_privacy');
        if (preferences && preferences.includes('2')) {
            // based on TrustArc cookie consent, cookie is allowed
            enableScripts();
            const initChatService = () => __awaiter(void 0, void 0, void 0, function* () {
                try {
                    yield waitForWindowVariable('embedded_svc', 10000);
                    initEmbedChat();
                }
                catch (e) {
                    console.error('Error: Chat service not available', e.message);
                    setLoadingChat(false);
                }
            });
            initChatService();
        }
        else {
            setHasBlockedByCookie(true);
        }
        setLoadingChat(false);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    const onStartChat = () => __awaiter(void 0, void 0, void 0, function* () {
        setLoadingChat(true);
        try {
            // if chat didn't start after a long wait
            // then we assume browser has blocked the script
            const blockedTimeOut = setTimeout(() => {
                if (!window.$Lightning) {
                    setHasBlockedByBrowser(true);
                    setLoadingChat(false);
                }
            }, 10000);
            yield window.embedded_svc.liveAgentAPI.startChat({
                extraPrechatInfo: extraPreChatInfo,
                extraPrechatFormDetails: getChatFormDetails(),
            });
            clearTimeout(blockedTimeOut);
            setHasBlockedByBrowser(false);
            setChatStart(true);
            setLoadingChat(false);
        }
        catch (e) {
            setLoadingChat(false);
            setHasBlockedByBrowser(true);
        }
    });
    const initEmbedChat = () => {
        const extraPreChatFormDetails = getChatFormDetails();
        try {
            initESW(extraPreChatFormDetails);
        }
        catch (e) {
            setChatServiceNotAvailable(true);
            console.log(e);
        }
    };
    const initESW = (extraPreChatFormDetails) => {
        window.embedded_svc.settings.displayHelpButton = false; //Or false
        window.embedded_svc.settings.language = 'en'; //For example, enter 'en' or 'en-US'
        window.embedded_svc.settings.enabledFeatures = ['LiveAgent'];
        window.embedded_svc.settings.entryFeature = 'LiveAgent';
        window.embedded_svc.settings.extraPrechatFormDetails = extraPreChatFormDetails;
        window.embedded_svc.settings.extraPrechatInfo = extraPreChatInfo;
        window.embedded_svc.init(host, sandBox, gslbBaseURL, key, appName, extra);
        // Fired when Embedded Service Chat has ended and the application is closed.
        window.embedded_svc.addEventHandler('afterDestroy', function (data) {
            setChatStart(false);
        });
        // if chat didn't initialized after a long wait then we assume
        // chat domains blocked by client network or browser
        const blockedByNetworkTimeOut = setTimeout(() => {
            if (!window.liveAgentDeployment) {
                setHasChatDomainsBlocked(true);
                setLoadingChat(false);
            }
        }, 10000);
        // After loading and setting step is complete we are good to call `bootstrapEmbeddedService`
        window.embedded_svc.addEventHandler('onSettingsCallCompleted', function (data) {
            setLoadingChat(false);
            clearTimeout(blockedByNetworkTimeOut);
        });
    };
    return {
        loadingChat,
        isChatStarted,
        hasChatDomainsBlocked,
        hasBlockedByBrowser,
        initEmbedChat,
        onStartChat,
        chatServiceNotAvailable,
        hasBlockedByCookie,
    };
};
